@@ -479,8 +479,8 @@ _edje_program_run(Edje *ed, Edje_Program *pr, Eina_Bool force, const char *ssig,
      }
    if ((recursions >= 64) || (recursion_limit))
      {
-	ERR("Programs recursing up to recursion limit of %i. Disabled.",
-	    64);
+	ERR("Programs recursing up to recursion limit of %i in '%s' with '%s', '%s' from '%s', '%s'. Disabled.",
+	    64, pr->name, ssig, ssrc, ed->path, ed->group);
 	recursion_limit = 1;
 	return;
      }
@@ -531,7 +531,7 @@ _edje_program_run(Edje *ed, Edje_Program *pr, Eina_Bool force, const char *ssig,
                               }
 
 			    if (rp->program)
-			      _edje_program_end(ed, rp->program);
+                              _edje_program_end(ed, rp->program);
 			    _edje_part_description_apply(ed, rp,
 							 rp->param1.description->state.name,
 							 rp->param1.description->state.value,
@@ -570,7 +570,7 @@ _edje_program_run(Edje *ed, Edje_Program *pr, Eina_Bool force, const char *ssig,
 		       if (rp)
 			 {
 			    if (rp->program)
-			      _edje_program_end(ed, rp->program);
+                              _edje_program_end(ed, rp->program);
 			    _edje_part_description_apply(ed, rp,
 							 pr->state,
 							 pr->value,
@@ -873,8 +873,11 @@ _edje_emit_full(Edje *ed, const char *sig, const char *src, void *data, void (*f
 {
    Edje_Message_Signal emsg;
    const char *sep;
+   Eina_Bool broadcast;
 
+   if (!ed->collection) return;
    if (ed->delete_me) return;
+   broadcast = ed->collection->broadcast_signal;
 
    sep = strchr(sig, EDJE_PART_PATH_SEPARATOR);
 
@@ -958,6 +961,7 @@ _edje_emit_full(Edje *ed, const char *sig, const char *src, void *data, void (*f
               if (!ed2) goto end;
 
               _edje_emit(ed2, newsig, src);
+	      broadcast = EINA_FALSE;
               break;
 
            case EDJE_PART_TYPE_EXTERNAL:
@@ -976,6 +980,7 @@ _edje_emit_full(Edje *ed, const char *sig, const char *src, void *data, void (*f
                    if (!ed2) goto end;
                    _edje_emit(ed2, newsig, src);
                 }
+	      broadcast = EINA_FALSE;
               break ;
 
            case EDJE_PART_TYPE_BOX:
@@ -988,11 +993,12 @@ _edje_emit_full(Edje *ed, const char *sig, const char *src, void *data, void (*f
                    ed2 = _edje_fetch(child);
                    if (!ed2) goto end;
                    _edje_emit(ed2, newsig, src);
+		   broadcast = EINA_FALSE;
                 }
               break ;
 
            default:
-              fprintf(stderr, "SPANK SPANK SPANK !!!\nYou should never be here !\n");
+              ERR("SPANK SPANK SPANK !!!\nYou should never be here !");
               break;
           }
      }
@@ -1012,7 +1018,10 @@ _edje_emit_full(Edje *ed, const char *sig, const char *src, void *data, void (*f
         emsg.data = NULL;
      }
 /* new sends code */
-   edje_object_message_send(ed->obj, EDJE_MESSAGE_SIGNAL, 0, &emsg);
+   if (broadcast)
+     edje_object_message_send(ed->obj, EDJE_MESSAGE_SIGNAL, 0, &emsg);
+   else
+     _edje_message_send(ed, EDJE_QUEUE_SCRIPT, EDJE_MESSAGE_SIGNAL, 0, &emsg);
 /* old send code - use api now
    _edje_message_send(ed, EDJE_QUEUE_SCRIPT, EDJE_MESSAGE_SIGNAL, 0, &emsg);
    EINA_LIST_FOREACH(ed->subobjs, l, obj)
@@ -1037,12 +1046,8 @@ _edje_emit_full(Edje *ed, const char *sig, const char *src, void *data, void (*f
 
 struct _Edje_Program_Data
 {
-#ifdef EDJE_PROGRAM_CACHE
   Eina_List     *matches;
-  int            matched;
-#endif
   Edje          *ed;
-  const char    *signal;
   const char    *source;
 };
 
@@ -1056,33 +1061,16 @@ static Eina_Bool _edje_glob_callback(Edje_Program *pr, void *dt)
      {
 	rp = _edje_real_part_get(data->ed, pr->filter.part ? pr->filter.part : data->source);
 	if (rp)
-	  exec = (rp->chosen_description->state.name == pr->filter.state);
+          exec = (rp->chosen_description->state.name == pr->filter.state);
      }
 
-#ifdef EDJE_PROGRAM_CACHE
-   data->matched++;
-#endif
+   pr->exec = exec;
 
-   if (exec)
-     _edje_program_run(data->ed, pr, 0, data->signal, data->source);
-
-   if (_edje_block_break(data->ed))
-     {
-#ifdef EDJE_PROGRAM_CACHE
-        eina_list_free(data->matches);
-        data->matches = NULL;
-#endif
-        return EINA_TRUE;
-     }
-
-#ifdef EDJE_PROGRAM_CACHE
    data->matches = eina_list_append(data->matches, pr);
-#endif
 
    return EINA_FALSE;
 }
 
-
 void
 _edje_callbacks_patterns_clean(Edje *ed)
 {
@@ -1127,7 +1115,7 @@ _edje_emit_handle(Edje *ed, const char *sig, const char *src,
    
    if (ed->collection && ed->L)
      _edje_lua2_script_func_signal(ed, sig, src);
-   
+
    if (ed->collection)
      {
 #ifdef EDJE_PROGRAM_CACHE
@@ -1156,11 +1144,11 @@ _edje_emit_handle(Edje *ed, const char *sig, const char *src,
 
 	     if (eina_hash_find(ec->prog_cache.no_matches, tmps))
 	       {
-		  done = 1;
+ 		  done = 1;
 	       }
 	     else if ((matches = eina_hash_find(ec->prog_cache.matches, tmps)))
 	       {
-		 EINA_LIST_FOREACH(matches, l, pr)
+                  EINA_LIST_FOREACH(matches, l, pr)
 		    {
                        Eina_Bool exec = EINA_TRUE;
 
@@ -1169,10 +1157,12 @@ _edje_emit_handle(Edje *ed, const char *sig, const char *src,
                             Edje_Real_Part *rp;
 
                             rp = _edje_real_part_get(ed, pr->filter.part ? pr->filter.part : src);
-                            if (rp)
+                            if (rp && !rp->program)
                               exec = (rp->chosen_description->state.name == pr->filter.state);
                          }
 
+                       pr->exec = exec;
+#if 0
                        if (exec)
                          {
                             _edje_program_run(ed, pr, 0, sig, src);
@@ -1181,7 +1171,19 @@ _edje_emit_handle(Edje *ed, const char *sig, const char *src,
                                  goto break_prog;
                               }
                          }
+#endif
 		    }
+
+                  EINA_LIST_FOREACH(matches, l, pr)
+                    if (pr->exec)
+                      {
+                         _edje_program_run(ed, pr, 0, sig, src);
+                         if (_edje_block_break(ed))
+                           {
+                              goto break_prog;
+                           }
+                      }
+
 		  done = 1;
 	       }
 	  }
@@ -1192,11 +1194,8 @@ _edje_emit_handle(Edje *ed, const char *sig, const char *src,
 
              data.ed = ed;
              data.source = src;
-             data.signal = sig;
-#ifdef EDJE_PROGRAM_CACHE
-	     data.matched = 0;
 	     data.matches = NULL;
-#endif
+
              if (ed->table_programs_size > 0)
                {
 		  const Eina_List *match;
@@ -1218,12 +1217,29 @@ _edje_emit_handle(Edje *ed, const char *sig, const char *src,
 							    ed->patterns.programs.exact_match);
 		  EINA_LIST_FOREACH(match, l, pr)
 		    _edje_glob_callback(pr, &data);
+
+#ifdef EDJE_PROGRAM_CACHE
+                  EINA_LIST_FOREACH(data.matches, l, pr)
+#else
+                  EINA_LIST_FREE(data.matches, pr)
+#endif
+                    {
+                       if (pr->exec)
+                         _edje_program_run(ed, pr, 0, sig, src);
+
+                       if (_edje_block_break(ed))
+                         {
+                            eina_list_free(data.matches);
+                            data.matches = NULL;
+                            goto break_prog;
+                         }
+                    }
                }
 
 #ifdef EDJE_PROGRAM_CACHE
 	     if (tmps)
 	       {
-		  if (data.matched == 0)
+		  if (data.matches == NULL)
 		    {
 		      if (!ec->prog_cache.no_matches)
 			ec->prog_cache.no_matches = eina_hash_string_superfast_new(NULL);
@@ -1273,7 +1289,7 @@ _edje_emit_cb(Edje *ed, const char *sig, const char *src, Edje_Message_Signal_Da
    if (ed->just_added_callbacks)
      _edje_callbacks_patterns_clean(ed);
 
-   ed->walking_callbacks = 1;
+   ed->walking_callbacks++;
 
    if (ed->callbacks)
      {
@@ -1305,14 +1321,15 @@ _edje_emit_cb(Edje *ed, const char *sig, const char *src, Edje_Message_Signal_Da
                {
                   escb->func(escb->data, ed->obj, sig, src);
                   if (_edje_block_break(ed))
-                    goto break_prog;
+                    break;
                }
           }
      }
    break_prog:
 
-   ed->walking_callbacks = 0;
-   if ((ed->delete_callbacks) || (ed->just_added_callbacks))
+   ed->walking_callbacks--;
+   if (!ed->walking_callbacks &&
+       ((ed->delete_callbacks) || (ed->just_added_callbacks)))
      {
 	ed->delete_callbacks = 0;
 	ed->just_added_callbacks = 0;
@@ -1621,6 +1638,7 @@ _edje_param_native_set(Edje_Real_Part *rp, const char *name, const Edje_External
 		  if (param->type != EDJE_EXTERNAL_PARAM_TYPE_DOUBLE)
 		    return EINA_FALSE;
 		  rp->drag->size.x = FROM_DOUBLE(CLAMP(param->d, 0.0, 1.0));
+                  rp->edje->recalc_call = 1;
 		  rp->edje->dirty = 1;
 #ifdef EDJE_CALC_CACHE
 		  rp->invalidate = 1;
@@ -1633,6 +1651,7 @@ _edje_param_native_set(Edje_Real_Part *rp, const char *name, const Edje_External
 		  if (param->type != EDJE_EXTERNAL_PARAM_TYPE_DOUBLE)
 		    return EINA_FALSE;
 		  rp->drag->size.y = FROM_DOUBLE(CLAMP(param->d, 0.0, 1.0));
+                  rp->edje->recalc_call = 1;
 		  rp->edje->dirty = 1;
 #ifdef EDJE_CALC_CACHE
 		  rp->invalidate = 1;
