@@ -10,7 +10,7 @@ multisense_sound_player_get(Edje_Multisense_Env *msenv)
    RemixEnv *env = msenv->remixenv;
    RemixPlugin *player_plugin;
    RemixBase *player;
-   
+
    player_plugin = remix_find_plugin(env, "alsa_snd_player");
    if (!player_plugin)
      {
@@ -23,7 +23,7 @@ multisense_sound_player_get(Edje_Multisense_Env *msenv)
 #endif
 
 EAPI Eina_Bool
-multisense_factory_init(Edje_Multisense_Env *env)
+multisense_factory_init(Edje_Multisense_Env *env __UNUSED__)
 {
 #ifdef HAVE_LIBREMIX
    remix_set_samplerate(env->remixenv, DEFAULT_SAMPLERATE);
